/* "Pomegranate" Virtual Boy Library
   version .90 (Alpha pre-release 15-Sept-2002)
   
   by frostgiant, portions based on code by David Tucker and
      Aleberto Covarrubias

   I've titled this library "Pomegranate" because the fruit is red and the
   VB is red. Just "VB Lib" would be too boring. */


#ifndef _VBLIB_H_
#define _VBLIB_H_

#ifndef _VBTYPES_H_
#define BYTE unsigned char
#define HWORD unsigned short
#define WORD unsigned long
#endif

#ifndef _VBADDR_H_
#define VIP_REGS(x)             *((HWORD*)(0x0005F800 + x))
#define SND_REGS(x)             *((BYTE*)(0x01000000 + x))
#define HW_REGS(x)              *((BYTE*)(0x02000000 + x))
#define	CharSeg0        0x6000
#define	CharSeg1        0xE000
#define	CharSeg2        0x16000
#define	CharSeg3        0x1E000
#define	ObjectSeg       0x3E000
#define BGMap(x)		(0x20000+(0x2000*x))
#define World(x)		(0x3D800+(x*0x20))
#endif

/* Macros for world manipulation */
#define WORLD_HEAD(n,head)      *(HWORD*)World(n)=head
#define WORLD_GSET(n,gx,gp,gy)  *(HWORD*)(World(n)+2)=gx;\
                                *(HWORD*)(World(n)+4)=gp;\
                                *(HWORD*)(World(n)+6)=gy;
#define WORLD_MSET(n,mx,mp,my)  *(HWORD*)(World(n)+8)=mx;\
                                *(HWORD*)(World(n)+10)=mp;\
                                *(HWORD*)(World(n)+12)=my;
#define WORLD_SIZE(n,w,h)       *(HWORD*)(World(n)+14)=w;\
                                *(HWORD*)(World(n)+16)=h;

/* Macro to set the brightness for the colors
   Note: color = a + b + c */     
#define SET_BRIGHT(a,b,c)       VIP_REGS(0x24)=a;       \
                                VIP_REGS(0x26)=b;       \
                                VIP_REGS(0x28)=c;


/* Macro to set the GPLT (BGMap palette) */
#define SET_GPLT(n,pal)         VIP_REGS(0x60+n)=pal;

/* Macro to set the JPLT (OBJ palette) */
#define SET_JPLT(n,pal)         VIP_REGS(0x68+n)=pal;

/* Keypad Definitions */
#define LOWBATT 0x01            /* Low Battery          */
#define K_A     0x04            /* A Button             */
#define K_B     0x08            /* B Button             */
#define K_TR    0x10            /* R Trigger            */
#define K_TL    0x20            /* L Trigger            */
#define K_RU    0x40            /* Right Pad, Up        */
#define K_RR    0x80            /* Right Pad, Right     */
#define K_LR    0x100           /* Left Pad,  Right     */
#define K_LL    0x200           /* Left Pad,  Left      */
#define K_LD    0x400           /* Left Pad,  Down      */
#define K_LU    0x800           /* Left Pad,  Up        */
#define K_ST    0x1000          /* Start Button         */
#define K_SL    0x2000          /* Select Button        */
#define K_RL    0x4000          /* Right Pad, Left      */
#define K_RD    0x8000          /* Right Pad, Down      */


/* Delay execution */
void delay(int count) {
        HWORD valueI=0;
        HWORD valueC=0;

        *(WORD*)0x0500FFD7=0;
        *(WORD*)0x0500FFDB=0;
        *(WORD*)0x0500FFDF=0;

        while(count) {
           valueI=*(HWORD*)0x0005F840;

           /* We cannot use a while loop here because it gets
              "optimized" out by the compiler */
           check:
           valueC=*(HWORD*)0x0005F840;
           if(valueI==valueC) { goto check; }

           count--;
        }        
}

/* Reads the keypad, returns the result in a HWORD */
HWORD readpad(void) {
        BYTE status;
        BYTE hibyte;
        BYTE lobyte;
        *(BYTE*)0x02000028=0x80;
        *(BYTE*)0x02000028=0x84;

        while(status) {
           status=*(BYTE*)0x2000028;
           status=status&0x02;
        }

        hibyte=*(BYTE*)0x02000014;
        lobyte=*(BYTE*)0x02000010;
        return (hibyte<<8)|lobyte;
} 

/* Turns the display on. This function is a bit of a mess, it needs to be
   cleaned up. */
void display_on(void) {
        HWORD tempH;
        WORD  tempW;
        *(WORD*)0x0500FFD7=0;
        *(HWORD*)0x0005F82A=0;
        tempH=*(HWORD*)0x0005F840;
        tempH=tempH&0xFFFF;
        tempH=tempH|2;
        *(HWORD*)0x0005F842=tempH;
        tempH=*(HWORD*)0x0005F820;
        tempH=tempH&0xFFFF;
        tempH=tempH|0x302;
        *(HWORD*)0x0005F822=tempH;
        *(HWORD*)0x0005F82E=0;
        tempH=*(HWORD*)0x0005F800;
        *(HWORD*)0x0005F804=tempH;
        tempH=*(HWORD*)0x0005F820;
        tempH=0x3C&tempH;
        tempW=*(WORD*)0x0500FFD7;
        tempW=tempW+0x14;
}

/* Call this after the display is on and you want the image to show up */
void display_show(void) {
          HWORD r6=0;
          HWORD r18=0;
          HWORD r29=0;

          *(WORD*)0x0500FFD7=0;
          *(WORD*)0x0500FFDB=0;

          while(r29-20<=0) {
             r6=15;
             r18=r29;
             *(HWORD*)0x0005F828=r18;
             r18=r29;
             r18=r18<<1;
             *(HWORD*)0x0005F824=r18;
             r29++;
         }
}

/* Initializes the pallettes and brightness */
void display_init()
{
        VIP_REGS(0x24) = 0x20;
        VIP_REGS(0x26) = 0x40;
        VIP_REGS(0x28) = 0x80;
	VIP_REGS(0x60) = 0xE4;
	VIP_REGS(0x62) = 0xE4;
	VIP_REGS(0x64) = 0xE4;
	VIP_REGS(0x65) = 0xE4;
	VIP_REGS(0x66) = 0xE4;
	VIP_REGS(0x6A) = 0xE4;
	VIP_REGS(0x6C) = 0xE4;
	VIP_REGS(0x6E) = 0xE4;
	VIP_REGS(0x70) = 0;
} 

#ifndef _VBHW_H_

/* This function is done in assembley */
void* memcpy(void* dest, const void* src, int count);

#endif


#endif
